<?php

/*
 * rdSimpleAuth Component
 *
 * @author		gwoo <gwoo@rd11.com>
 * @version		1.1.4.3104
 * @license		MIT
 *
 */
class rdSimpleAuthComponent extends Object {
	var $components = array (
		'Session'
	);
	/**
	* the redirect if auth fails.
	*
	* @var string
	* @access public
	*/
	var $loginPage = '/users/login';

	/**
	* all roles available
	*
	* @var array
	* @access public
	*/
	var $roles = array (
		'Admin' => 'Admin'
	);

	/**
	* if role assigned to the logged in user matches admins array
	*
	* @var array
	* @access public
	*/
	var $admins = array (
		'Admin' => 'Admin'
	);

	/**
	* allowed roles
	*
	* @var array
	* @access public
	*/
	var $allows = array (
		'Admin' => 'Admin'
	);

	/**
	* id of the logged in user
	*
	* @var unknown_type
	* @access private
	*/

	var $id = null;

	/**
	* username of the logged in user
	*
	* @var string
	* @access private
	*/
	var $username = null;

	/**
	* role assigned to the logged in user
	*
	* @var string
	* @access private
	*/
	var $role = null;

	/**
	* if admins passes then set admin var
	*
	* @var bool
	* @access private
	*/
	var $admin = null;

	/**
	* if role passes users set allowed
	*
	* @var boolean
	* @access private
	*/
	var $allow = null;

	/**
	* Valid if role, username , and id exist.
	*
	* @var string
	* @access public
	*/
	var $valid = false;

	/**
	* Error messages to be displayed if the user is short of access for the requested action.
	*
	* @var string
	* @access private
	*/
	var $errors = null;

	/**
	* Uses callback to automatically check rights
	* @return redirect if check fails
	*/
	function startup(& $controller) {
		$this->set();
		if (!isset ($controller->params['url']['url'])) {
			$controller->params = array (
				'url' => array (
					'url' => ''
				)
			);
		}

		if (!strstr($controller->params['url']['url'], substr($this->loginPage, 1))) {
			$vars = get_class_vars($controller->name . 'Controller');

			if (empty ($vars['rdAccess'])) {
				$vars = get_class_vars('AppController');
			}

			if (empty ($vars['rdAccess'])) {
				$vars = array (
					'rdAccess' => ''
				);
			}

			if ($this->check($controller->action, $vars['rdAccess']) == false) {
				$this->Session->write('rdAuth.redirect', $controller->params['url']['url']);
				$controller->redirect($this->loginPage);
				exit;
			/**
			 * Modified by Tarique Sani <tarique@sanisoft.com> to cope with
			 * login being persistently shown once displayed
			 */
			}elseif (is_array($vars['rdAccess']) && array_key_exists($controller->action, $vars['rdAccess'])) {
				if ($this->Session->check('rdAuth.redirect')) {
					$redirect = $this->Session->read('rdAuth.redirect');
					$this->Session->del('rdAuth.redirect');
					$controller->redirect($redirect);

				}
			}
		}
	}

	/**
	* Function to check the session and return local vars
	*
	* @param string $data used for login method
	* @return errors
	*/
	function set($data = '') {
		if ($data) {
			$this->Session->write('rdAuth.id', $data['id']);
			$this->Session->write('rdAuth.username', $data['username']);
			$this->Session->write('rdAuth.role', $data['role']);
		}
		if ($this->Session->check('rdAuth') && $this->Session->valid('rdAuth')) {
			$this->id = $this->Session->read('rdAuth.id');
			$this->username = $this->Session->read('rdAuth.username');
			$this->role = $this->Session->read('rdAuth.role');
			if (in_array($this->role, $this->admins)) {
				$this->admin = 1;
			}
			if (in_array($this->role, $this->allows)) {
				$this->allow = 1;
			}
			if ($this->id && $this->username && $this->role) {
				$this->valid = 1;
			}
		}
		elseif ($this->Session->error()) {
			return $this->Session->error();
		}
	}

	/**
	* logout method deletes session
	*
	* @return errors
	*/
	function logout() {
		$this->Session->del('rdAuth.id');
		$this->Session->del('rdAuth.username');
		$this->Session->del('rdAuth.role');
		$this->Session->del('rdAuth');
		if ($this->Session->error()) {
			return $this->Session->error();
		}
	}

	/**
	* Function to check the access for the action based on the access list
	*
	* @param string $action The action for which we need to check the access
	* @param array $access Access array for the controller's actions
	* @return boolean
	*/
	function check($action, $access = '') {
		if ($access == '*') {
			if ($this->valid && $this->allow) {
				return true;
			}
			return false;
		} else
			if (is_array($access) && array_key_exists($action, $access)) {
				if ($this->role) {
					if (in_array($this->role, $access[$action])) {
						return true;
					} else {
						return false;
					}
				} else
					if ($this->id) {
						if (in_array($this->id, $access[$action])) {
							return true;
						} else {
							return false;
						}
					} else
						if ($this->username) {
							if (in_array($this->username, $access[$action])) {
								return true;
							} else {
								return false;
							}
						} else {
							return false;
						}
			}
		return true;
	}

	/**
	* Function to check if a access is authorized on a row in the database
	* your table would have a field that contains the role just like the user table
	*
	* @param string $item The field for which we need to check the access
	* @return boolean
	*/
	function perm($item = null) {
		if (isset ($item) && isset ($this->role)) {
			if (in_array($this->role, $this->allows)) {
				$flipped = array_flip(array_values($this->allows));
				if (isset ($flipped[$item]) && $flipped[$item] >= $flipped[$this->role]) {
					return true;
				}
			}
		}
		return false;
	}
}
?>